@@ -6,6 +6,7 @@
 !  2002/05/19  ݹ  yt_module ⥸塼̾ѹ.
 !                            ʻ̤ɬפ뤿. 
 !      2002/06/10  ݹ  ݥ뼧춭롼ɲ
+!      2002/11/10  ݹ  ¶֤Ǥζ롼ɲ
 !
 !
 !      ǡ index
@@ -59,9 +60,32 @@ module wt_module
   public wt_RadRot_xyz_xyz, wt_RadRotRot_xyz_xyz_xyz
   public wt_Potential2vector
   public wt_VGradV
-  public wt_Boundaries, wt_TorBoundaries, wz_LaplaPol2Pol_wz
+
+  public wt_Boundaries, wt_TorBoundaries
   public wt_TormagBoundaries, wt_PolmagBoundaries
 
+  public wt_BoundariesTau, wt_TorBoundariesTau
+  public wt_TormagBoundariesTau, wt_PolmagBoundariesTau
+
+  public wt_BoundariesGrid, wt_TorBoundariesGrid, wz_LaplaPol2Pol_wz
+  public wt_TormagBoundariesGrid, wt_PolmagBoundariesGrid
+
+  interface wt_Boundaries
+     module procedure wt_BoundariesTau
+  end interface
+
+  interface wt_TorBoundaries
+     module procedure wt_TorBoundariesTau
+  end interface
+
+  interface wt_TorMagBoundaries
+     module procedure wt_TorMagBoundariesTau
+  end interface
+
+  interface wt_PolMagBoundaries
+     module procedure wt_PolMagBoundariesTau
+  end interface
+
   integer            :: im=64, jm=32, km=16  ! ʻ(, , ư)
   integer            :: nm=21, lm=16         ! ȿ(ʿ, ư)
   real(8)            :: ri=0.0, ro=1.0       ! ⳰Ⱦ
@@ -512,8 +536,8 @@ module wt_module
 
   !---------------  -----------------
 
-    subroutine wt_Boundaries(wt,values,cond)   ! ǥꥯ, Υޥ
-      ! Chebyshev ֤ǤζŬ
+    subroutine wt_BoundariesTau(wt,values,cond)   ! ǥꥯ, Υޥ
+      ! Chebyshev ֤ǤζŬ(ˡ)
 
       real(8), dimension((nm+1)*(nm+1),0:lm),intent(inout)      :: wt
               ! ŬѤǡ. 줿֤ͤ. 
@@ -530,9 +554,9 @@ module wt_module
 
       if (.not. present(cond)) then
          if (present(values)) then
-            call at_boundaries_DD(wt,values)
+            call at_BoundariesTau_DD(wt,values)
          else
-            call at_boundaries_DD(wt)
+            call at_BoundariesTau_DD(wt)
          endif
          return
       endif
@@ -540,35 +564,91 @@ module wt_module
       select case(cond)
       case ('NN')
          if (present(values)) then
-            call at_boundaries_NN(wt,values)
+            call at_BoundariesTau_NN(wt,values)
          else
-            call at_boundaries_NN(wt)
+            call at_BoundariesTau_NN(wt)
          endif
       case ('DN')
          if (present(values)) then
-            call at_boundaries_DN(wt,values)
+            call at_BoundariesTau_DN(wt,values)
          else
-            call at_boundaries_DN(wt)
+            call at_BoundariesTau_DN(wt)
          endif
       case ('ND')
          if (present(values)) then
-            call at_boundaries_ND(wt,values)
+            call at_BoundariesTau_ND(wt,values)
          else
-            call at_boundaries_ND(wt)
+            call at_BoundariesTau_ND(wt)
          endif
       case ('DD')
          if (present(values)) then
-            call at_boundaries_DD(wt,values)
+            call at_BoundariesTau_DD(wt,values)
          else
-            call at_boundaries_DD(wt)
+            call at_BoundariesTau_DD(wt)
          endif
       case default
-         call msgdmp('E','wt_Boundaries','B.C. not supported')
+         call msgdmp('E','wt_BoundariesTau','B.C. not supported')
       end select
 
-    end subroutine wt_Boundaries
+    end subroutine wt_BoundariesTau
 
-    subroutine wt_TorBoundaries(wt_TORPOT,cond) ! ȥݥƥ󥷥붭
+    subroutine wt_BoundariesGrid(wt,values,cond)   ! ǥꥯ, Υޥ
+      ! ¶֤ǤζŬ
+
+      real(8), dimension((nm+1)*(nm+1),0:lm),intent(inout)      :: wt
+              ! ŬѤǡ. 줿֤ͤ. 
+
+      real(8), dimension((nm+1)*(nm+1),2), intent(in), optional :: values
+              ! Ǥ / ʬۤʿڥȥѴΤͿ. 
+              ! ά/ 0 Ȥʤ. 
+
+      character(len=2), intent(in), optional             :: cond
+              ! . ά 'DD'
+              !   DD : ξüǥꥯ
+              !   DN,ND : ǥꥯ/Υޥ
+              !   NN : ξüΥޥ
+
+      if (.not. present(cond)) then
+         if (present(values)) then
+            call at_boundariesGrid_DD(wt,values)
+         else
+            call at_boundariesGrid_DD(wt)
+         endif
+         return
+      endif
+
+      select case(cond)
+      case ('NN')
+         if (present(values)) then
+            call at_BoundariesGrid_NN(wt,values)
+         else
+            call at_BoundariesGrid_NN(wt)
+         endif
+      case ('DN')
+         if (present(values)) then
+            call at_BoundariesGrid_DN(wt,values)
+         else
+            call at_BoundariesGrid_DN(wt)
+         endif
+      case ('ND')
+         if (present(values)) then
+            call at_BoundariesGrid_ND(wt,values)
+         else
+            call at_BoundariesGrid_ND(wt)
+         endif
+      case ('DD')
+         if (present(values)) then
+            call at_BoundariesGrid_DD(wt,values)
+         else
+            call at_BoundariesGrid_DD(wt)
+         endif
+      case default
+         call msgdmp('E','wt_BoundariesGrid','B.C. not supported')
+      end select
+
+    end subroutine wt_BoundariesGrid
+
+    subroutine wt_TorBoundariesTau(wt_TORPOT,cond) ! ȥݥƥ󥷥붭
       ! Chebyshev ֤ǤζŬ
       !  ȥݥƥ󥷥 \psi ζŬ
       ! Ǵ
@@ -647,7 +727,96 @@ module wt_module
       wt_torpot(:,lm)   = 0
       wt_torpot = lusolve(alu,kp,wt_TORPOT)
 
-    end subroutine wt_TorBoundaries
+    end subroutine wt_TorBoundariesTau
+
+    subroutine wt_TorBoundariesGrid(wt_TORPOT,cond) ! ȥݥƥ󥷥붭
+      ! ¶֤ǤζŬ
+      !  ȥݥƥ󥷥 \psi ζŬ
+      ! Ǵ
+      !    \psi = 0  at boundaries
+      ! Ϥʤ
+      !    \DP{\psi/r}{r} = 0  at boundaries
+
+      real(8), dimension((nm+1)*(nm+1),0:lm),intent(inout)      :: wt_TORPOT
+              ! ŬѤǡ. 줿֤ͤ. 
+
+      character(len=2), intent(in), optional  :: cond
+              ! 凉å. ά 'RR'
+              !   RR : ξüǴ
+              !   RF,FR : Ǵ/Ϥʤ
+              !   FF : ξüϤʤ
+
+
+      real(8), dimension((nm+1)*(nm+1),0:km):: wz_TORPOT
+      real(8), dimension(:,:), allocatable  :: alu
+      integer, dimension(:), allocatable    :: kp
+      real(8), dimension(0:lm,0:lm)         :: tt_data
+      real(8), dimension(0:lm,0:km)         :: tz_data
+      logical                               :: rigid1, rigid2   ! 
+
+      logical :: first = .true.
+      integer  :: l
+      save     :: alu, kp, first
+
+      if (.not. present(cond)) then
+         rigid1=.TRUE. ; rigid2=.TRUE.
+      else
+         select case (cond)
+         case ('RR')
+            rigid1 = .TRUE.  ; rigid2 = .TRUE.
+         case ('RF')
+            rigid1 = .TRUE.  ; rigid2 = .FALSE.
+         case ('FR')
+            rigid1 = .FALSE. ; rigid2 = .TRUE.
+         case ('FF')
+            rigid1 = .FALSE. ; rigid2 = .FALSE.
+         case default
+            call msgdmp('E','wt_TorBoundaries','B.C. not supported')
+         end select
+      endif
+
+      if ( first ) then
+         first = .false.
+
+         if ( lm /= km ) then
+            call msgdmp('E','TorBoundariesGrid', &
+             'Chebyshev truncation and number of grid points should be same.')
+         endif
+
+         allocate(alu(0:km,0:lm),kp(0:lm))
+
+         tt_data = 0
+         do l=0,lm
+            tt_data(l,l)=1
+         enddo
+         tz_data = az_at(tt_data)
+         alu = transpose(tz_data)       ! ΰǤͤϤΤޤ
+
+         if ( rigid1 ) then
+            tz_data = az_at(tt_data)    ! ǴǤ 0
+         else
+            tz_data = az_at(at_dr_at(at_az( &
+                 az_at(tt_data)/spread(z_rad,1,lm+1))))
+         endif
+         alu(0,:) = tz_data(:,0)
+
+         if ( rigid2 ) then
+            tz_data = az_at(tt_data)    ! ǴǤ 0
+         else
+            tz_data = az_at(at_dr_at(at_az( &
+                 az_at(tt_data)/spread(z_rad,1,lm+1))))
+         endif
+         alu(km,:)   = tz_data(:,km)
+
+         call ludecomp(alu,kp)
+      endif
+      wz_TorPot       = wz_wt(wt_TorPot)
+      wz_TorPot(:,0)  = 0
+      wz_TorPot(:,km) = 0
+
+      wt_torpot = lusolve(alu,kp,wz_TorPot)
+
+    end subroutine wt_TorBoundariesGrid
 
     function wz_LaplaPol2Pol_wz(wz,cond)        ! ^2 ݥ
                                                 ! ǥեȤǴ
@@ -758,7 +927,7 @@ module wt_module
 
     end function wz_LaplaPol2Pol_wz
 
-    subroutine wt_TormagBoundaries(wt_TOR) ! ȥݥƥ󥷥붭
+    subroutine wt_TormagBoundariesTau(wt_TOR) ! ȥݥƥ󥷥붭
       ! Chebyshev ֤ǤζŬ
       ! ȥݥƥ󥷥 \psi ζ(ŵƳ)
       ! ¦
@@ -811,9 +980,71 @@ module wt_module
       wt_TOR(:,lm)   = 0.0
       wt_TOR = lusolve(alu,kp,wt_TOR)
 
-    end subroutine wt_TormagBoundaries
+    end subroutine wt_TormagBoundariesTau
 
-    subroutine wt_PolmagBoundaries(wt_POL) ! ݥݥƥ󥷥붭
+    subroutine wt_TormagBoundariesGrid(wt_TOR) ! ȥݥƥ󥷥붭
+      ! ľ¶֤ǤζŬ
+      ! ȥݥƥ󥷥 \psi ζ(ŵƳ)
+      ! ¦
+      !    wt_psi = 0   at the outer boundary
+      ! ¦
+      !    wt_psi = 0       at the inner boundary
+      ! 
+      ! wt_Boundaries бǽ, ΤӺƤ
+
+      real(8), dimension((nm+1)*(nm+1),0:lm),intent(inout)   :: wt_TOR
+              ! ŬѤǡ. 줿֤ͤ. 
+
+      real(8), dimension(:,:,:), allocatable  :: alu
+      integer, dimension(:,:), allocatable    :: kp
+
+      real(8), dimension(:,:), allocatable    :: tt_I
+      real(8), dimension(:,:), allocatable    :: tz_PSI
+      real(8), dimension((nm+1)*(nm+1),0:km)  :: wz_TOR
+
+      logical :: first = .true.
+      integer  :: l, n
+      save     :: alu, kp, first
+
+      if ( first ) then
+         first = .false.
+
+         if ( lm /= km ) then
+            call msgdmp('E','TorMagBoundariesGrid', &
+             'Chebyshev truncation and number of grid points should be same.')
+         endif
+
+         allocate(alu((nm+1)*(nm+1),0:km,0:lm),kp((nm+1)*(nm+1),0:lm))
+         allocate(tt_I(0:lm,0:lm),tz_PSI(0:lm,0:km))
+
+         tt_I = 0.0
+         do l=0,lm
+            tt_I(l,l)=1
+         enddo
+         do n=1,(nm+1)*(nm+1)
+            alu(n,:,:) = transpose(az_at(tt_I))   ! ΰͤΤޤ.
+         enddo
+
+         ! ŵƳ
+         tz_PSI = az_at(tt_I)
+
+         do n=1,(nm+1)*(nm+1)
+            alu(n,0,:) = tz_PSI(:,0)
+            alu(n,km,:)   = tz_PSI(:,km)
+         enddo
+         call ludecomp(alu,kp)
+
+         deallocate(tt_I,tz_PSI)
+      endif
+      
+      wz_TOR       = wz_wt(wt_TOR)
+      wz_TOR(:,0)  = 0.0
+      wz_TOR(:,km) = 0.0
+      wt_TOR = lusolve(alu,kp,wz_TOR)
+
+    end subroutine wt_TormagBoundariesGrid
+
+    subroutine wt_PolmagBoundariesTau(wt_POL) ! ݥݥƥ󥷥붭
       ! Chebyshev ֤ǤζŬ
       ! ݥݥƥ󥷥 \psi ζ(ŵƳ)
       ! ¦
@@ -867,6 +1098,69 @@ module wt_module
       wt_POL(:,lm)   = 0.0
       wt_POL = lusolve(alu,kp,wt_POL)
 
-    end subroutine wt_PolmagBoundaries
+    end subroutine wt_PolmagBoundariesTau
+
+    subroutine wt_PolmagBoundariesGrid(wt_POL) ! ݥݥƥ󥷥붭
+      ! ľ¶֤ǤζŬ
+      ! ݥݥƥ󥷥 \psi ζ(ŵƳ)
+      ! ¦
+      !    \DP{wt_psi}{r} + (l+1) wt_psi/r = 0   at the outer boundary
+      ! ¦
+      !    \DP{wt_psi}{r} - l wt_psi/r = 0       at the inner boundary
+
+      real(8), dimension((nm+1)*(nm+1),0:lm),intent(inout)   :: wt_POL
+              ! ŬѤǡ. 줿֤ͤ. 
+
+      real(8), dimension(:,:,:), allocatable  :: alu
+      integer, dimension(:,:), allocatable    :: kp
+
+      real(8), dimension(:,:), allocatable    :: tt_I
+      real(8), dimension(:,:), allocatable    :: tz_PSI
+      real(8), dimension(:,:), allocatable    :: tz_DPSIDR
+      real(8), dimension((nm+1)*(nm+1),0:km)  :: wz_POL
+
+      logical :: first = .true.
+      integer  :: l, n, nn(2)
+      save     :: alu, kp, first
+
+      if ( first ) then
+         first = .false.
+
+         if ( lm /= km ) then
+            call msgdmp('E','PolMagBoundariesGrid', &
+             'Chebyshev truncation and number of grid points should be same.')
+         endif
+
+         allocate(alu((nm+1)*(nm+1),0:lm,0:lm),kp((nm+1)*(nm+1),0:lm))
+         allocate(tt_I(0:lm,0:lm),tz_PSI(0:lm,0:km),tz_DPSIDR(0:lm,0:km))
+
+         tt_I = 0.0
+         do l=0,lm
+            tt_I(l,l)=1
+         enddo
+         do n=1,(nm+1)*(nm+1)
+            alu(n,:,:) = transpose(az_at(tt_I))  ! ΰͤ¸
+         enddo
+
+         ! ŵƳ
+         tz_PSI = az_at(tt_I)
+         tz_DPSIDR = az_at(at_dr_at(tt_I))
+
+         do n=1,(nm+1)*(nm+1)
+            nn=nm_l(n)
+            alu(n,0,:)  = tz_DPSIDR(:,0) + (nn(1)+1) * tz_PSI(:,0)/z_RAD(0)
+            alu(n,km,:) = tz_DPSIDR(:,km) - nn(1) * tz_PSI(:,km)/z_RAD(km)
+         enddo
+         call ludecomp(alu,kp)
+
+         deallocate(tt_I,tz_PSI,tz_DPSIDR)
+      endif
+
+      wz_POL       = wz_wt(wt_POL)
+      wz_POL(:,0)  = 0.0
+      wz_POL(:,km) = 0.0
+      wt_POL = lusolve(alu,kp,wz_POL)
+
+    end subroutine wt_PolmagBoundariesGrid
 
   end module wt_module
